/* **********************************************************************************
	Definition der im Modul verwendeten Variablen, Eventhandler und Funktionen.
	Auslesen der Eingabe Datumswerte im manuellen Modus. Feststellen von Zeitzone
	und DST sowie Berechnung und Ausgabe des:
	- Julianischen Datums und dessen gregorianische Rckrechnung
	- Modified Julianischen Datums und dessen Rckrechnung
	- UT = Universal Time Ausgabe
	- UTC = Universal Time Coordinated Ausgabe
 ********************************************************************************** */

var d = null;
var actualDate = null;
var calcDate = null;
var year = "";
var month = "";
var day = "";
var hour = "";
var minute = "";
var second = "";
var millisecond = "";
var JD0 = 0;
var JD = 0;
var MJD = 0;
// Das Datum vom 1. Januar des aktuellen Jahres
var jan = null;
// Das Datum vom 1. Juli des aktuellen Jahres
var jul = null;
// Zeitzonenwert = Timezone + DST
// Standard ist Winterzeit, also GMT + 1 h 
var TIMEZONE = 0;
// Differenz zwischen Juli und Januar. Wenn DST dann ist die Differnz 1,
// ansonsten 0
var diff = 0;


/* **********************************************************************************
	Objekte fr die Eingabe Datums- und Zeitfelder. Dienen der Bedienung
	und Ausgabe der Ergebnisse.
 ********************************************************************************** */

var objDate = document.getElementById("date");
var objTime = document.getElementById("time");
var objSec = document.getElementById("second");
var objMilliSec = document.getElementById("msecond");
var output = document.getElementById("kontrolldatum");
var outputLT = document.getElementById("LT");
var outputUT = document.getElementById("UT");
var outputJD1 = document.getElementById("JD1");
var outputMJD = document.getElementById("MJD");
var outputDateMJD = document.getElementById("DateMJD");
var outputDate1 = document.getElementById("DateJD1");
var objTimezone = document.getElementById("Zeitzone");
var objDST = document.getElementById("DST");
var objCalc = document.getElementById("calc");
var divAutomatic = document.getElementById("automatic");


// Wartefunktion die verwendet werden kann.
// Die Funktion, in der sie verwendet wird muss vom Typ async sein:
//
// async function myFunction(){....}
//
function Sleep(milliseconds) {
 return (new Promise(resolve => setTimeout(resolve, milliseconds)));
}


// Aktuelles Datum und Uhrzeit in den Pickern setzen
setActualDateAndTime();

/* ******************************************************
	Die Funktion fllt eine Zeichenketten mit Zahl vorne
	mit n-Zeichen auf.
	Z.B. 1 -> 01
	izahl = 		Die Zahl ohne fhrende Nullen
	ianzStellen = 	Die gewnschte Anzahl von Stellen
	sFuelzeichen =	Das gewnschet fhrende Zeichen
	
	Rckgabewert =	Z.B. "01" oder "0001"
   *************************************************** */
function LPad( iZahl, iAnzStellen, sFuellzeichen )
{
    var sZahl = iZahl + "";
    while( sZahl.length < iAnzStellen )
        sZahl = sFuellzeichen + sZahl;
    return sZahl;
}


// Die Funktion setzt das aktuelle Datum und Uhrzeit in den Pickern.
// 
function setActualDateAndTime(){
	var h = "";
	var m = "";
	var s = "";
	var ms = "";
	// Datumspicker mit aktuellem Datum versehen.
	d = new Date();				// Aktuelles Datum holen
	objDate.valueAsDate = d;	// und dem Datumspicker zuweisen
	// Stunde und Minute mit "0" auffllen und Time Picker
	// damit initialisieren.
	if ((h = d.getHours()) < 10)
		h = LPad(h, 2, "0");
	if ((m = d.getMinutes()) < 10)
		m = LPad(m, 2, "0");
	objTime.value = h + ":" + m;
	// Sekunden mit "0" auffllen und Sekunden Picker
	// damit initialisieren.
	s = d.getSeconds();
	objSec.value = s;
	// Millisekunden und Millisekunden Picker
	// damit initialisieren.
	ms = d.getMilliseconds();
	objMilliSec.value = ms;
}


// Bei Programmstart DST und Timezone ermitteln sowie
// die Select Boxen vorbelegen.
//
calcDSTandTimezone();


// Event Listener beim Klick auf den Button "Berechnen"
// startet die Verarbeitung.
//
objCalc.addEventListener("click", getDateValues);


/* **********************************************************************************
	Eventhandler fr die Eingabe Datums- und Zeitfelder. Bei Klick darauf werden
	die Ausgabefelder gelscht.
 ********************************************************************************** */

// Bei Klick auf das Datums Input Element wird das Kontrolldatum
// gelscht.
//
objDate.addEventListener("click", function () {
	eraseOutputFields();
});


// Bei Klick auf das Time Input Element wird das Kontrolldatum
// gelscht.
//
objTime.addEventListener("click", function () {
	eraseOutputFields();
});


// Bei Klick auf das Second Input Element wird das Kontrolldatum
// gelscht.
//
objSec.addEventListener("click", function () {
	eraseOutputFields();
});


// Bei Klick auf das Milli-Second Input Element wird das Kontrolldatum
// gelscht.
//
objMilliSec.addEventListener("click", function () {
	eraseOutputFields();
});


// Lscht die Ausgabefelder.
// 
function eraseOutputFields(){
	output.value = "";
	outputLT.value = "";
	outputUT.value = "";
	outputJD1.value = "";
	outputDate1.value = "";
	outputMJD.value = "";
	outputDateMJD.value = "";
	document.getElementById("mstimelaps").value = "";
	document.getElementById("msvelosity").value = "";
	document.getElementById("msprogresstime").value = "";
}


// Bei einem Klick auf den Radio Button "Automatisch"
// werden die Datums- und Time Picker ausgeschaltet.
//
function disableDateTimePicker(){
	objDate.disabled = true;
	objTime.disabled = true;
	objSec.disabled = true;
	objMilliSec.disabled = true;
}


// Bei einem Klick auf den Radio Button "Manuell"
// werden die Datums- und Time Picker eingeschaltet.
//
function enableDateTimePicker(){
	objDate.disabled = false;
	objTime.disabled = false;
	objSec.disabled = false;
	objMilliSec.disabled = false;
}



/* **********************************************************************************
	Eventhandler fr die manuelle oder automatische Verarbeitung bei einem Klick auf
	die Radio Buttons Manuell oder Automatisch.
 ********************************************************************************** */

// Eventhandler bwi Klick auf den Radio Button "Manuell".
// 
var objCalcManual = document.getElementById("calcManual");
objCalcManual.addEventListener("click", function () {
	// Stoppt den Worker fr das Datum Overlay.
	//
	if (wAutomaticTime) stopWorker();
	// Wenn der Worker fr die Berechnung "Ab aktuellem Datum" endlos luft,
	// dann stoppe Ihn
	if (workerActualEndless) stopWorkerActualDateEndless();
	objCalcManual.checked = "checked";			// Schaltet Radion Button Manuell "Ein".
	objCalcAutomatic.checked = "";				// Button "Automatisch" wird ausgeschaltet.
	divAutomatic.style.visibility = "hidden";	// Overlay wird geschllossen.
	objCalc.disabled = false;					// Button "Berechnen" des Hauptformulars ist "Ein".
	setActualDateAndTime();						// Aktuelles Datum und Uhrzeit als Startdatum setzen.
	eraseOutputFields();						// Die Ausgabefelder lschen.
	enableDateTimePicker();						// Die obigen Datumspicker einschalten.
	document.getElementById("msvelosity").value = "N/A";
	document.getElementById("mstimelaps").value = "N/A";
	document.getElementById("msprogresstime").value = "N/A";
});


// Eventhandler fr die Auswahl des Radio Buttons "Automatisch".
//
var objCalcAutomatic = document.getElementById("calcAutomatic");
objCalcAutomatic.addEventListener("click", function () {
	// Wenn der Worker fr die Berechnung "Ab aktuellem Datum" endlos luft,
	// dann stoppe Ihn
	if (workerActualEndless) stopWorkerActualDateEndless();
	objCalcUntilDateAutomatic.disabled = false;
	objCalcManual.checked = "";					// Manuell Ausschalten.
	objCalcAutomatic.checked = "checked";		// Automatisch markieren.
	divAutomatic.style.visibility = "visible";	// Das Overlay fr die Einstellungen Anzeigen.
	objCalc.disabled = true;					// "Berechnen" Button im Hauptformular Ausgrauen.
	setActualDateAndTimeAutomatic();			// Aktuelles Datum und Uhrzeit im Overlay setzen.
	disableDateTimePicker();					// Die obigen Datumsoicker ausschalten.
});




// <select> ... </select> HTML Element fr Dropdowns automatisch
// selektieren.
// arrSelect = Objekt auf das Select Element
// sItem     = Wert = value, der in <option> steht und ausgewhlt wird
//
function SelectItemInSelect(arrSelect, sItem) {
	for (k=0; k < arrSelect.options.length; k++) {
		if (arrSelect.options[k].value==sItem) {
			arrSelect.options[k].selected = true;
		}
	}
}


/* **********************************************************************************
	DST und Timezone ermitteln und die Select Boxen mit den Werten einstellen.
 ********************************************************************************** */


// Daylight Savings Time (Sommerzeit) ermitteln.
// Gibt true zurck wenn DST, ansonsten false
//
function isDST (d) {
	var ret = false;
	var JAN = 0; JUL = 0;
	// Das Datum vom 1. Januar des aktuellen Jahres
	var jan = new Date(d.getFullYear(), 0, 1);
	// Das Datum vom 1. Juli des aktuellen Jahres
	var jul = new Date(d.getFullYear(), 6, 1);

	JAN = jan.getTimezoneOffset();
	JUL = jul.getTimezoneOffset();
						
	// Zeitzone nach Osten
	if (JAN < 0 && JUL < 0){
		// Im Juli kleinerer Wert, also Sommerzeit
		if (JUL < JAN)
			ret = true;
		// Winterzeit
		else if (JUL == JAN)
			ret = false;
	// Zeitzone nach Westen
	} else if (JAN > 0 && JUL > 0){
			// Im Juli kleinerer Wert, also Sommerzeit
			if (JUL < JAN){
				ret = true;
			// Winterzeit
			}else if (JUL == JAN){
				ret = false;
			// Zeitzone Greenwich
			}else if (JAN == 0 && JUL == 0){
				ret = false;
			// Nicht logischer Wert
			}else{
				ret = false;
			}
	}
						
	return (ret);
}


// Bei Programmstart wird die Funktion gerufen und ermittelt
// Sommer/Winterzeit wie die Timezone. Stellt die Select Boxen
// daruf ein.
//
function calcDSTandTimezone(){
	// Aktuelles Datum ermitteln
	d = new Date();
	// Das Datum vom 1. Januar des aktuellen Jahres
	jan = new Date(d.getFullYear(), 0, 1);
	// Das Datum vom 1. Juli des aktuellen Jahres
	jul = new Date(d.getFullYear(), 6, 1);
	// Zeitzonenwert = Timezone + DST
	// Standard ist Winterzeit, also GMT + 1 h 
	TIMEZONE = 0;
	// Differenz zwischen Juli und Januar. Wenn DST dann ist die Differnz 1,
	// ansonsten 0
	diff = 0;
	// Differnz zwischen Juli und Januar des Jahres ermitteln
	diff = jul.getTimezoneOffset()/60 - jan.getTimezoneOffset()/60;
	TIMEZONE = diff;
	// und wenn DST, dann zur Timezone dazuaddieren.
	if  ( isDST(d) ){
		// Sommerzeit automatisch eintragen
		SelectItemInSelect(objDST, 2);
		// Nach Osten ist der Wert negativ und nach Westen positiv
		if ( TIMEZONE < 0 ){
			TIMEZONE -= 1;
			// Zeitzone automatisch eintragen
			SelectItemInSelect(objTimezone, Math.abs(TIMEZONE + 1));
		}else{
			TIMEZONE += 1;
			// Zeitzone automatisch eintragen
			SelectItemInSelect(objTimezone, --Math.abs(TIMEZONE - 1));
		}
	}else{
		// Winterzeit automatisch eintragen
		SelectItemInSelect(objDST, 1);
		// Zeitzone automatisch eintragen
		SelectItemInSelect(objTimezone, Math.abs(TIMEZONE));
	}
}


/* **********************************************************************************
	Bei Berechnungsstart wird getDateValues() zuerst gerufen,danach getTimeValues().
 ********************************************************************************** */

// Die eingegebenen Datumswerte ermitteln und in Variablen speichern.
//
function getDateValues(){
	year = d.getFullYear();
	month = d.getMonth() + 1;
	day = d.getDate();
	// Zeitwerte ermitteln
	getTimeValues();
}


// Die Eingegebenen Zeitfelder auslesen und verarbeiten.
//
function getTimeValues(){
	var ot = objTime.value.split(":");
	hour = ot[0];
	minute = ot[1]
	second = objSec.value;
	millisecond = objMilliSec.value;
	var LT = 0;
	var UTeqGMT = 0;
	var UThour = 0;
	var Date1 = null;
	
	LT = Number(hour) + (minute/60.) + (second/3600.) + (millisecond/1000./3600.);
	// Um 0 Uhr Mitternacht muss fr UT mit 24 gerechnet werden
	if (hour == 0) UThour = 22; else UThour = hour - Math.abs(TIMEZONE);
	// UT nach DST und Zeitzone
	UTeqGMT = LT - Math.abs(TIMEZONE);
	// Ausgabe der lokalen Zeit = UTC
	outputLT.value = Math.trunc(hour) + ":" + minute + ":" + second + "." + millisecond;
	// Ausgabe der UT = GMT
	outputUT.value = Math.trunc(UThour) + ":" + minute + ":" + second + "." + millisecond;
	// Date Objekt mit den eingebennen Datumswerten
	calcDate = new Date(year, month - 1, day, hour, minute, second, millisecond);
	// Ausgabe des gesamten Datums mit Zeit zur Kontrolle
	output.value = calcDate.toLocaleString() + "." + millisecond;
	// Ausgabe der Julianischen Datumsergebnisse nach Ephemeriden Rechnung von Oliver Montenbruck
	JD1 = calculateJD(day,month,year, UTeqGMT);
	outputJD1.value = "";
	outputJD1.value = JD1;
	// Rckrechnung von JD nach Datum und Uhrzeit
	Date1 = calculateDate(JD1);
	if ( Date1[3] + Math.abs(TIMEZONE) == 24) {
		Date1[3] -= 22;
		Date1[0] += 1;
	} else
		Date1[3] += Math.abs(TIMEZONE);
	outputDate1.value = Date1[0] + "." + Date1[1] + "." + Date1[2] + " - " + Date1[3] + ":" + Date1[4] + ":" + Date1[5] + "." + Date1[6];
	// Rckrechnung von MJD nach Datum und Uhrzeit, aber nur ab 1900
	if ( parseInt(Date1[2]) >= 1900 ){
		// Ausgabe des Modified Julianischen Datums
		MJD = calculateMJD(JD1);
		outputMJD.value = MJD;
		Date1 = calculateDateFromMJD(MJD);
		if ( Date1[3] + Math.abs(TIMEZONE) == 24) {
			Date1[3] -= 22;
			Date1[0] += 1;
		} else
			Date1[3] += Math.abs(TIMEZONE);
		outputDateMJD.value = Date1[0] + "." + Date1[1] + "." + Date1[2] + " - " + Date1[3] + ":" + Date1[4] + ":" + Date1[5] + "." + Date1[6];
	}else{
		outputMJD.value = "N/A";
		outputDateMJD.value = "N/A";
	}
}